==> streetlevel-0.6.2.tar.gz: streetlevel-0.6.2/streetlevel/lookaround/panorama.py <==
from __future__ import annotations

import base64
import math
from dataclasses import dataclass
from datetime import datetime
from enum import Enum

from streetlevel.lookaround.proto import MuninViewState_pb2


class CoverageType(Enum):
    """
    Coverage type of a Look Around panorama.
    """
    CAR = 2
    """
    The panorama was taken by a car.
    """
    BACKPACK = 3
    """
    The panorama was taken by a backpack.
    """


@dataclass
class LookaroundPanorama:
    """
    Metadata of a Look Around panorama.
    """

    id: int
    """The pano ID."""
    build_id: int
    """
    An additional parameter required for requesting the imagery. Every time Apple publishes or updates 
    a bunch of panoramas, they are assigned a build ID to act as a revision number.
    """

    lat: float
    """Latitude of the panorama's location."""
    lon: float
    """Longitude of the panorama's location."""

    heading: float = None
    """
    Heading in radians, where 0° is north, 90° is west, 180° is south, 270° is east.
    
    Pitch and roll are not yet available in this library.
    """

    coverage_type: CoverageType = None
    """Whether the coverage was taken by car or by backpack."""
    date: datetime = None
    """Capture date and time of the panorama (in UTC, not local time)."""

    elevation: float = None
    """Elevation at the capture location in meters."""

    has_blurs: bool = None
    """Whether something in this panorama, typically a building, has been blurred."""

    def permalink(self: LookaroundPanorama, heading: float = 0.0, pitch: float = 0.0, radians: bool = False) -> str:
        """
        Creates a link which will open a panorama at this location in Apple Maps. Linking to a specific panorama
        by its ID does not appear to be possible.

        On non-Apple devices, the link will redirect to Google Maps.

        :param heading: *(optional)* Initial heading of the viewport. Defaults to 0°.
        :param pitch: *(optional)* Initial pitch of the viewport. Defaults to 0°.
        :param radians: *(optional)* Whether angles are in radians. Defaults to False.
        :return: An Apple Maps URL.
        """
        if radians:
            heading = math.degrees(heading)
            pitch = math.degrees(pitch)
        mvs = MuninViewState_pb2.MuninViewState()
        mvs.viewState.latitude = self.lat
        mvs.viewState.longitude = self.lon
        mvs.viewState.yaw = heading
        mvs.viewState.pitch = -pitch
        mvs_base64 = base64.b64encode(mvs.SerializeToString())
        return f"https://maps.apple.com/?ll={self.lat},{self.lon}&_mvs={mvs_base64.decode()}"

    def __repr__(self):
        return str(self)

    def __str__(self):
        return f"{self.id}/{self.build_id} ({self.lat:.5f}, {self.lon:.5f})"

==> streetlevel-0.6.3.tar.gz: streetlevel-0.6.3/streetlevel/lookaround/panorama.py <==
from __future__ import annotations

import base64
import math
from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Tuple

from streetlevel.lookaround import geo
from streetlevel.lookaround.proto import MuninViewState_pb2


class CoverageType(Enum):
    """
    Coverage type of a Look Around panorama.
    """
    CAR = 2
    """
    The panorama was taken by a car.
    """
    BACKPACK = 3
    """
    The panorama was taken by a backpack.
    """


@dataclass
class LookaroundPanorama:
    """
    Metadata of a Look Around panorama.
    """

    id: int
    """The pano ID."""
    build_id: int
    """
    An additional parameter required for requesting the imagery. Every time Apple publishes or updates 
    a bunch of panoramas, they are assigned a build ID to act as a revision number.
    """

    lat: float
    """Latitude of the panorama's location."""
    lon: float
    """Longitude of the panorama's location."""

    coverage_type: CoverageType = None
    """Whether the coverage was taken by car or by backpack."""
    date: datetime = None
    """Capture date and time of the panorama (in UTC, not local time)."""

    has_blurs: bool = None
    """Whether something in this panorama, typically a building, has been blurred."""

    raw_orientation: Tuple[int, int, int] = None
    """The raw yaw, pitch, and roll values returned by the API."""

    raw_altitude: int = None
    """The raw altitude value returned by the API."""

    tile: Tuple[int, int, int] = None
    """The tile this panorama is located on."""

    _heading: float = None
    _pitch: float = None
    _roll: float = None
    _elevation: float = None
    _altitude: float = None

    def _set_altitude_and_elevation(self):
        if not self._elevation:
            self._altitude, self._elevation = \
                geo.convert_altitude(self.raw_altitude, self.lat, self.lon, self.tile[0], self.tile[1])

    def _set_orientation(self):
        if not self._heading:
            self._heading, self._pitch, self._roll = \
                geo.convert_pano_orientation(self.lat, self.lon, *self.raw_orientation)

    @property
    def elevation(self) -> float:
        """Elevation at the capture location in meters."""
        self._set_altitude_and_elevation()
        return self._elevation

    @property
    def heading(self) -> float:
        """
        Heading in radians, where 0° is north, 90° is west, 180° is south, 270° is east.
        """
        self._set_orientation()
        return self._heading

    @property
    def pitch(self) -> float:
        """Pitch offset for upright correction of the panorama, in radians."""
        self._set_orientation()
        return self._pitch

    @property
    def roll(self) -> float:
        """Roll offset for upright correction of the panorama, in radians."""
        self._set_orientation()
        return self._roll

    def permalink(self: LookaroundPanorama, heading: float = 0.0, pitch: float = 0.0, radians: bool = False) -> str:
        """
        Creates a link which will open a panorama at this location in Apple Maps. Linking to a specific panorama
        by its ID does not appear to be possible.

        On non-Apple devices, the link will redirect to Google Maps.

        :param heading: *(optional)* Initial heading of the viewport. Defaults to 0°.
        :param pitch: *(optional)* Initial pitch of the viewport. Defaults to 0°.
        :param radians: *(optional)* Whether angles are in radians. Defaults to False.
        :return: An Apple Maps URL.
        """
        if radians:
            heading = math.degrees(heading)
            pitch = math.degrees(pitch)
        mvs = MuninViewState_pb2.MuninViewState()
        mvs.viewState.latitude = self.lat
        mvs.viewState.longitude = self.lon
        mvs.viewState.yaw = heading
        mvs.viewState.pitch = -pitch
        mvs_base64 = base64.b64encode(mvs.SerializeToString())
        return f"https://maps.apple.com/?ll={self.lat},{self.lon}&_mvs={mvs_base64.decode()}"

    def __repr__(self):
        return str(self)

    def __str__(self):
        return f"{self.id}/{self.build_id} ({self.lat:.5f}, {self.lon:.5f})"
